<?php
//
// Windows Server Config Wizard
// Copyright (c) 2020-2021 Nagios Enterprises, LLC. All rights reserved.
// 

include_once(dirname(__FILE__) . '/../configwizardhelper.inc.php');

nsclient_configwizard_init();

function nsclient_configwizard_init()
{
    $name = "nsclient";
    $args = array(
        CONFIGWIZARD_NAME => $name,
        CONFIGWIZARD_VERSION => "1.0.2",
        CONFIGWIZARD_TYPE => CONFIGWIZARD_TYPE_MONITORING,
        CONFIGWIZARD_DESCRIPTION => _("Monitor Windows using the legacy monitoring agent. (Deprecated)"),
        CONFIGWIZARD_DISPLAYTITLE => _("NSClient++ (Legacy)"),
        CONFIGWIZARD_FUNCTION => "nsclient_configwizard_func",
        CONFIGWIZARD_PREVIEWIMAGE => "nsclient.png",
        CONFIGWIZARD_FILTER_GROUPS => array('windows'),
        CONFIGWIZARD_REQUIRES_VERSION => 5600
    );
    register_configwizard($name, $args);
}

/**
 * @param string $mode
 * @param null   $inargs
 * @param        $outargs
 * @param        $result
 *
 * @return string
 */
function nsclient_configwizard_func($mode = "", $inargs = null, &$outargs = null, &$result = null)
{
    $wizard_name = $inargs['wizard'];

    $agents_url = "http://nsclient.org/download/";
    $agent32_stable_url = "https://assets.nagios.com/downloads/nagiosxi/agents/NSClient++/NSCP-0.4.4.23-Win32.msi";
    $agent64_stable_url = "https://assets.nagios.com/downloads/nagiosxi/agents/NSClient++/NSCP-0.4.4.23-x64.msi";

    // Initialize return code and output
    $result = 0;
    $output = "";

    // Initialize output args - pass back the same data we got
    $outargs[CONFIGWIZARD_PASSBACK_DATA] = $inargs;

    switch ($mode) {
        case CONFIGWIZARD_MODE_GETSTAGE1HTML:

            $address = grab_array_var($inargs, "ip_address", "");

            $output = '
<h5 class="ul">' . _('Host Information') . '</h5>
<table class="table table-condensed table-no-border table-auto-width">
    <tr>
        <td class="vt">
            <label>' . _('IP Address') . ':</label>
        </td>
        <td>
            <input type="text" size="40" name="ip_address" id="ip_address" value="' . encode_form_val($address) . '" class="form-control" autocomplete="ip_address">
            <div class="subtext">' . _('The IP address of the Windows system you\'d like to monitor.') . '</div>
        </td>
    </tr>
</table>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE1DATA:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "ip_address", "");
            $address = nagiosccm_replace_user_macros($address);

            // Check for errors
            $errors = 0;
            $errmsg = array();

            if (have_value($address) == false)
                $errmsg[$errors++] = _("No address specified.");
            else if (!valid_ip($address))
                $errmsg[$errors++] = _("Invalid IP address.");

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }

            break;

        case CONFIGWIZARD_MODE_GETSTAGE2HTML:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "ip_address");
            $port = grab_array_var($inargs, "port", "12489");
            $hostname = grab_array_var($inargs, "hostname", @gethostbyaddr($address));
            $password = grab_array_var($inargs, "password", "");

            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial", "");
            if ($services_serial != "")
                $services = json_decode(base64_decode($services_serial), true);
            if (!is_array($services)) {
                $services_default = array(
                    "ping" => 1,
                    "cpu" => 1,
                    "memory" => 1,
                    "uptime" => 1,
                    "disk" => 1,
                );
                $services = grab_array_var($inargs, "services", $services_default);
            }

            $serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");
            if ($serviceargs_serial != "")
                $serviceargs = json_decode(base64_decode($serviceargs_serial), true);

            if (!is_array($serviceargs)) {
                $serviceargs_default = array(

                    "memory_warning" => 80,
                    "memory_critical" => 90,

                    "cpu_warning" => 80,
                    "cpu_critical" => 90,

                    "processstate" => array(),
                    "servicestate" => array(),
                    "counter" => array(),
                );
                for ($x = 0; $x < 5; $x++) {
                    $serviceargs_default["disk_warning"][$x] = 80;
                    $serviceargs_default["disk_critical"][$x] = 95;
                    $serviceargs_default["disk"][$x] = ($x == 0) ? "C" : "";
                }
                for ($x = 0; $x < 4; $x++) {
                    if ($x == 0) {
                        $serviceargs_default['processstate'][$x]['process'] = 'explorer.exe';
                        $serviceargs_default['processstate'][$x]['name'] = 'Explorer';
                        $services["processstate"][$x] = ""; // defaults for checkboxes, enter on to be checked by default
                    } else {
                        $serviceargs_default['processstate'][$x]['process'] = '';
                        $serviceargs_default['processstate'][$x]['name'] = '';
                        $services["processstate"][$x] = ""; // defaults for checkboxes, enter on to be checked by default
                    }
                }

                for ($x = 0; $x < 4; $x++) {
                    if ($x == 0) {
                        $serviceargs_default['servicestate'][$x]['service'] = "W3SVC";
                        $serviceargs_default['servicestate'][$x]['name'] = "IIS Web Server";
                        $services["servicestate"][$x] = ""; // defaults for checkboxes, enter on to be checked by default
                    } else if ($x == 1) {
                        $serviceargs_default['servicestate'][$x]['service'] = "MSSQLSERVER";
                        $serviceargs_default['servicestate'][$x]['name'] = "SQL Server";
                        $services["servicestate"][$x] = ""; // defaults for checkboxes, enter on to be checked by default
                    } else {
                        $serviceargs_default['servicestate'][$x]['service'] = "";
                        $serviceargs_default['servicestate'][$x]['name'] = "";
                        $services["servicestate"][$x] = ""; // defaults for checkboxes, enter on to be checked by default
                    }
                }
                for ($x = 0; $x < 6; $x++) {
                    if ($x == 0) {
                        $serviceargs_default['counter'][$x]['counter'] = "\\\\Paging File(_Total)\\\\% Usage";
                        $serviceargs_default['counter'][$x]['name'] = "Page File Usage";
                        $serviceargs_default['counter'][$x]['format'] = "Paging File usage is %.2f %%";
                        $serviceargs_default['counter'][$x]['warning'] = "70";
                        $serviceargs_default['counter'][$x]['critical'] = "90";
                        $services["counter"][$x] = ""; // defaults for checkboxes, enter on to be checked by default
                    } else if ($x == 1) {
                        $serviceargs_default['counter'][$x]['counter'] = "\\\\Server\\\\Errors Logon";
                        $serviceargs_default['counter'][$x]['name'] = "Logon Errors";
                        $serviceargs_default['counter'][$x]['format'] = "Login Errors since last reboot is %.f";
                        $serviceargs_default['counter'][$x]['warning'] = "2";
                        $serviceargs_default['counter'][$x]['critical'] = "20";
                        $services["counter"][$x] = ""; // defaults for checkboxes, enter on to be checked by default
                    } else if ($x == 2) {
                        $serviceargs_default['counter'][$x]['counter'] = "\\\\Server Work Queues(0)\\\\Queue Length";
                        $serviceargs_default['counter'][$x]['name'] = "Server Work Queues";
                        $serviceargs_default['counter'][$x]['format'] = "Current work queue (an indication of processing load) is %.f ";
                        $serviceargs_default['counter'][$x]['warning'] = "4";
                        $serviceargs_default['counter'][$x]['critical'] = "7";
                        $services["counter"][$x] = ""; // defaults for checkboxes, enter on to be checked by default
                    } else {
                        $serviceargs_default['counter'][$x]['counter'] = "";
                        $serviceargs_default['counter'][$x]['name'] = "";
                        $serviceargs_default['counter'][$x]['format'] = "";
                        $serviceargs_default['counter'][$x]['warning'] = "";
                        $serviceargs_default['counter'][$x]['critical'] = "";
                        $services["counter"][$x] = ""; // defaults for checkboxes, enter on to be checked by default
                    }
                }

                $serviceargs = grab_array_var($inargs, "serviceargs", $serviceargs_default);
            }

            $output = '
<input type="hidden" name="ip_address" value="' . encode_form_val($address) . '">

<h5 class="ul">' . _('Host Details') . '</h5>
<table class="table table-condensed table-no-border table-auto-width">
    <tr>
        <td>
            <label>' . _('IP Address') . ':</label>
        </td>
        <td>
            <input type="text" size="40" name="ip_address" id="ip_address" value="' . encode_form_val($address) . '" class="form-control" disabled>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <label>' . _('Host Name') . ':</label>
        </td>
        <td>
            <input type="text" size="20" name="hostname" id="hostname" value="' . encode_form_val($hostname) . '" class="form-control">
            <div class="subtext">' . _('The name you\'d like to have associated with this Windows system') . '.</div>
        </td>
    </tr>
</table>

<h5 class="ul">' . _('NSClient++ Agent') . '</h5>

<p>' . _('You\'ll need to install an agent on the Windows system in order to monitor it.  For security purposes, it is recommended to use a password with the agent.') . '</p>

<div class="alert alert-info" style="margin-bottom: 0; width: 50%; min-width: 300px;">
    ' . _("This config wizard is <b>deprecated</b>. We recommend using the") . ' <a href="monitoringwizard.php?wizard=windowsserver&nextstep=2&nsp='.get_nagios_session_protector_id().'">' . _("Windows Server") . "</a> " . _("or") . ' <a href="monitoringwizard.php?wizard=windowsdesktop&nextstep=2&nsp='.get_nagios_session_protector_id().'">' . _("Windows Desktop") . "</a> " . _("configuration wizard instead.") . '
</div>

<table class="table table-condensed table-no-border table-auto-width">
    <tr>
        <td>
            <label>' . _('Agent Download') . ':</label>
        <td>
            <div class="pad-t5">
                <table class="table table-condensed table-no-border table-auto-width">
                    <thead>
                        <tr>
                            <th>' . _('32-Bit Agent') . '</th>
                            <th>' . _('64-Bit Agent') . '</th>
                        </tr>
                    </thead>
                    <tbody>
                        <tr>
                            <td>
                                <a href="' . $agent32_stable_url . '"><img src="' . theme_image("download.png") . '"></a> <a href="' . $agent32_stable_url . '"><b>' . _('Download') . ' v0.4.4 (32bit)</b></a>
                            </td>
                            <td>
                                <a href="' . $agent64_stable_url . '"><img src="' . theme_image("download.png") . '"></a> <a href="' . $agent64_stable_url . '"><b>' . _('Download') . ' v0.4.4 (64bit)</b></a>
                            </td>
                        </tr>
                    </tbody>
                </table>
            </div>
        </td>
    </tr>
    <tr>
        <td valign="top">
            <label>' . _('Agent Password') . ':</label>
        </td>
        <td>
            <input type="text" size="20" name="password" id="password" value="' . encode_form_val($password) . '" class="form-control">
            <div class="subtext">' . _('Valid characters include') . ': <b>a-zA-Z0-9 .\:_-@</b></div>
        </td>
    </tr>
</table>

<h5 class="ul">' . _('Server Metrics') . '</h5>
<p>' . _('Specify which services you\'d like to monitor for the system') . '.</p>
<table class="table table-no-border table-auto-width">
    <tr>
        <td>
            <input type="checkbox" id="p" class="checkbox" name="services[ping]"  ' . is_checked(checkbox_binary($services["ping"]), "1") . '>
        </td>
        <td>
            <label class="normal" for="p">
                <b>Ping</b><br>
                ' . _('Monitors the server with an ICMP ping.  Useful for watching network latency and general uptime.') . '
            </label>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <input type="checkbox" id="cpu" class="checkbox" name="services[cpu]" ' . is_checked(checkbox_binary($services["cpu"]), "1") . '>
        </td>
        <td>
            <label class="normal" for="cpu">
                <b>' . _('CPU') . '</b><br>
                ' . _('Monitors the CPU (processor usage) on the server.') . '
            </label>
            <div class="pad-t5">
                <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="2" name="serviceargs[cpu_warning]" value="' . encode_form_val($serviceargs["cpu_warning"]) . '" class="form-control condensed"> % &nbsp;<label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="2" name="serviceargs[cpu_critical]" value="' . encode_form_val($serviceargs["cpu_critical"]) . '" class="form-control condensed"> %
            </div>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <input type="checkbox" id="mu" class="checkbox" name="services[memory]" ' . is_checked(checkbox_binary($services["memory"]), "1") . '>
        </td>
        <td>
            <label class="normal" for="mu">
                <b>' . _('Memory Usage') . '</b><br>
                ' . _('Monitors the memory usage on the server.') . '
            </label>
            <div class="pad-t5">
                <label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="2" name="serviceargs[memory_warning]" value="' . encode_form_val($serviceargs["memory_warning"]) . '" class="form-control condensed">% &nbsp;<label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="2" name="serviceargs[memory_critical]" value="' . encode_form_val($serviceargs["memory_critical"]) . '" class="form-control condensed"> %
            </div>
        </td>
    </tr>
    <tr>
        <td>
            <input type="checkbox" id="up" class="checkbox" name="services[uptime]" ' . is_checked(checkbox_binary($services["uptime"]), "1") . '>
        </td>
        <td>
            <label class="normal" for="up">
                <b>'._('Uptime').'</b><br>
                ' . _('Monitors the uptime on the server.') . '
            </label>
        </td>
    </tr>
    <tr>
        <td class="vt">
            <input type="checkbox" id="du" class="checkbox" name="services[disk]" ' . is_checked(checkbox_binary($services["disk"]), "1") . '>
        </td>
        <td>
            <label class="normal" for="du">
                <b>'._('Disk Usage').'</b><br>
                ' . _('Monitors disk usage on the server.') . '
            </label>
            <div class="pad-t5">
                <table class="adddeleterow table table-condensed table-no-border table-auto-width" style="margin-bottom: 10px;">';

            for ($x = 0; $x < count($serviceargs["disk"]); $x++) {
                $checkedstr = "";
                if ($x == 0)
                    $checkedstr = "checked";
                $output .= '<tr>';
                $output .= '<td><label>'._('Drive').':</label> <select name="serviceargs[disk][' . $x . ']" class="form-control condensed">';
                $output .= '<option value=""></option>';
                for ($y = 0; $y < 26; $y++) {
                    $selected = "";
                    $diskname = chr(ord('A') + $y);
                    $selected = is_selected($serviceargs["disk"][$x], $diskname);
                    $output .= '<option value="' . $diskname . '" ' . $selected . '>' . $diskname . ':</option>';
                }
                $output .= '</select></td>';
                $output .= '<td><label><img src="'.theme_image('error.png').'" class="tt-bind" title="'._('Warning Threshold').'"></label> <input type="text" size="2" name="serviceargs[disk_warning][' . $x . ']" value="' . encode_form_val($serviceargs["disk_warning"][$x]) . '" class="form-control condensed"> % &nbsp;<label><img src="'.theme_image('critical_small.png').'" class="tt-bind" title="'._('Critical Threshold').'"></label> <input type="text" size="2" name="serviceargs[disk_critical][' . $x . ']" value="' . encode_form_val($serviceargs["disk_critical"][$x]) . '" class="form-control condensed"> %</td>';
                $output .= '</tr>';
            }
            $output .= '
                </table>
            </div>
        </td>
    </tr>
</table>

<h5 class="ul">' . _('Services') . '</h5>
<p>' . _('Specify any services that should be monitored to ensure they\'re in a running state.') . '</p>
<table class="adddeleterow table table-condensed table-no-border table-auto-width" style="margin-bottom: 10px;">
    <tr>
        <th></th>
        <th>' . _('Windows Service') . '</th>
        <th>' . _('Display Name') . '</th>
    </tr>
    ';
            for ($x = 0; $x < count($serviceargs['servicestate']); $x++) {

                $servicestring = encode_form_val($serviceargs['servicestate'][$x]['service']);
                $servicename = encode_form_val($serviceargs['servicestate'][$x]['name']);
                $is_checked = isset($services['servicestate'][$x])
                    ? is_checked($services['servicestate'][$x]) : '';

                $output .= '<tr><td><input type="checkbox" class="checkbox" name="services[servicestate][' . $x . ']" ' . $is_checked . '></td><td><input type="text" size="15" name="serviceargs[servicestate][' . $x . '][service]" value="' . $servicestring . '" class="form-control"></td><td><input type="text" size="20" name="serviceargs[servicestate][' . $x . '][name]" value="' . $servicename . '" class="form-control"></td></tr>';
            }
            $output .= '
</table>
<div style="height: 20px;"></div>

<h5 class="ul">' . _('Processes') . '</h5>
<p>' . _('Specify any processes that should be monitored to ensure they\'re running.') . '</p>
<table class="adddeleterow table table-condensed table-no-border table-auto-width" style="margin-bottom: 10px;">
    <tr>
        <th></th>
        <th>' . _('Windows Process') . '</th>
        <th>' . _('Display Name') . '</th>
    </tr>
    ';
            for ($x = 0; $x < count($serviceargs['processstate']); $x++) {

                $processstring = encode_form_val($serviceargs['processstate'][$x]['process']);
                $processname = encode_form_val($serviceargs['processstate'][$x]['name']);
                $is_checked = isset($services['processstate'][$x])
                    ? is_checked($services['processstate'][$x]) : '';

                $output .= '<tr><td><input type="checkbox" class="checkbox" name="services[processstate][' . $x . ']"  ' . $is_checked . '></td><td><input type="text" size="15" name="serviceargs[processstate][' . $x . '][process]" value="' . $processstring . '" class="form-control"></td><td><input type="text" size="20" name="serviceargs[processstate][' . $x . '][name]" value="' . $processname . '" class="form-control"></td></tr>';
            }
            $output .= '
</table>
<div style="height: 20px;"></div>

<h5 class="ul">' . _('Performance Counters') . '</h5>
<p>' . _('Specify any performance counters that should be monitored.') . '</p>
<table class="adddeleterow table table-condensed table-no-border table-auto-width" style="margin-bottom: 10px;">
    <tr>
        <th></th>
        <th>' . _('Performance Counter') . '</th>
        <th>' . _('Display Name') . '</th>
        <th>' . _('Counter Output Format') . '</th>
        <th>' . _('Warning') . '</th>
        <th>' . _('Critical') . '</th>
    </tr>
    ';

            for ($x = 0; $x < count($serviceargs['counter']); $x++) {

                $counterstring = encode_form_val($serviceargs['counter'][$x]['counter']);
                $countername = encode_form_val($serviceargs['counter'][$x]['name']);
                $counterformat = encode_form_val($serviceargs['counter'][$x]['format']);
                $warnlevel = encode_form_val($serviceargs['counter'][$x]['warning']);
                $critlevel = encode_form_val($serviceargs['counter'][$x]['critical']);
                $is_checked = isset($services['counter'][$x])
                    ? is_checked($services['counter'][$x]) : '';

                $output .= '<tr><td><input type="checkbox" class="checkbox" name="services[counter][' . $x . ']"  ' . $is_checked . '></td><td><input type="text" size="25" name="serviceargs[counter][' . $x . '][counter]" value="' . $counterstring . '" class="form-control"></td><td><input type="text" size="20" name="serviceargs[counter][' . $x . '][name]" value="' . $countername . '" class="form-control"></td><td><input type="text" size="25" name="serviceargs[counter][' . $x . '][format]" value="' . $counterformat . '" class="form-control"></td><td><input type="text" size="2" name="serviceargs[counter][' . $x . '][warning]" value="' . $warnlevel . '" class="form-control"></td><td><input type="text" size="2" name="serviceargs[counter][' . $x . '][critical]" value="' . $critlevel . '" class="form-control"></td></tr>';
            }

            $output .= '
</table>
<div style="height: 20px;"></div>';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE2DATA:

            // Get variables that were passed to us
            $address = grab_array_var($inargs, "ip_address");
            $hostname = grab_array_var($inargs, "hostname");
            $password = grab_array_var($inargs, "password");
            $password_t = "";

            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial");
            if ($services_serial != "")
                $services = json_decode(base64_decode($services_serial), true);
            else
                $services = grab_array_var($inargs, "services");

            $serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial");
            if ($serviceargs_serial != "")
                $serviceargs = json_decode(base64_decode($serviceargs_serial), true);
            else
                $serviceargs = grab_array_var($inargs, "serviceargs");

            // Handle name and display name spaces
            foreach ($services as $servicename => $value) {
                if (in_array($servicename, array("servicestate", "processstate", "counter"))) {
                    foreach ($value as $k => $val) {
                        if ($val == "on") {
                            // trim spaces front and back
                            $serviceargs[$servicename][$k]["name"] = trim($serviceargs[$servicename][$k]["name"]);

                            // convert multiple internal spaces to one
                            preg_replace('/\s\s+/', ' ', $serviceargs[$servicename][$k]["name"]);
                        }
                    }
                }
            }

            // Check for errors
            $errors = 0;
            $errmsg = array();

            if (is_valid_host_name($hostname) == false)
                $errmsg[$errors++] = _("Invalid host name.");

            // Require a service discription - allowing no description breaks config
            foreach ($services as $servicename => $value) {
                if (in_array($servicename, array("servicestate", "processstate", "counter"))) {
                    foreach ($value as $k => $val) {
                        if ($val == "on") {
                            if ($serviceargs[$servicename][$k]["name"] == "") {
                                if (array_key_exists("service", $serviceargs[$servicename][$k])) {
                                    $errmsg[$errors++] = _("You must select a service display name for each service. Create display name for service: " . $serviceargs[$servicename][$k]["service"]);
                                }

                                if (array_key_exists("process", $serviceargs[$servicename][$k])) {
                                    $errmsg[$errors++] = _("You must select a service display name for each service. Create display name for process: " . $serviceargs[$servicename][$k]["process"]);
                                }

                                if (array_key_exists("counter", $serviceargs[$servicename][$k])) {
                                    $errmsg[$errors++] = _("You must select a service display name for each service. Create display name for performance counter: " . $serviceargs[$servicename][$k]["counter"]);
                                }
                            }
                        }
                    }
                }
            }

            if (preg_match("/[^a-zA-Z0-9 \.\\\\:_-@]/", $password)) {
                if (!preg_match('/\$USER[0-9]+\$/', $password) && !preg_match('/\$[a-zA-Z]+\$/', $password)) { // Check for user macros
                    $errmsg[$errors++] = _("Password contains invalid characters.");
                }
            }

            if ($errors > 0) {
                $outargs[CONFIGWIZARD_ERROR_MESSAGES] = $errmsg;
                $result = 1;
            }

            break;


        case CONFIGWIZARD_MODE_GETSTAGE3HTML:

            // get variables that were passed to us
            $address = grab_array_var($inargs, "ip_address");
            $hostname = grab_array_var($inargs, "hostname");
            $password = grab_array_var($inargs, "password");

            $services = "";
            $services_serial = grab_array_var($inargs, "services_serial");
            if ($services_serial != "")
                $services = json_decode(base64_decode($services_serial), true);
            else
                $services = grab_array_var($inargs, "services");

            $serviceargs = "";
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial");
            if ($serviceargs_serial != "")
                $serviceargs = json_decode(base64_decode($serviceargs_serial), true);
            else
                $serviceargs = grab_array_var($inargs, "serviceargs");

            $output = '
            
        <input type="hidden" name="ip_address" value="' . encode_form_val($address) . '">
        <input type="hidden" name="hostname" value="' . encode_form_val($hostname) . '">
        <input type="hidden" name="password" value="' . encode_form_val($password) . '">
        <input type="hidden" name="services_serial" value="' . base64_encode(json_encode($services)) . '">
        <input type="hidden" name="serviceargs_serial" value="' . base64_encode(json_encode($serviceargs)) . '">
';
            break;

        case CONFIGWIZARD_MODE_VALIDATESTAGE3DATA:

            break;

        case CONFIGWIZARD_MODE_GETFINALSTAGEHTML:

            $output = '
            <p>' . _('Dont forget to download and install the Windows Agent on the target server!') . '</p>
            <table class="table table-condensed table-no-border table-auto-width">
            <tr>
                <td>
                    <label>' . _('Agent Download') . ':</label>
                <td>
                    <div class="pad-t5">
                        <table class="table table-condensed table-no-border table-auto-width">
                            <thead>
                                <tr>
                                    <th>' . _('32-Bit Agent') . '</th>
                                    <th>' . _('64-Bit Agent') . '</th>
                                </tr>
                            </thead>
                            <tbody>
                                <tr>
                                    <td>
                                        <a href="' . $agent32_stable_url . '"><img src="' . theme_image("download.png") . '"></a> <a href="' . $agent32_stable_url . '"><b>' . _('Download') . ' v0.4.4 (32bit)</b></a>
                                    </td>
                                    <td>
                                        <a href="' . $agent64_stable_url . '"><img src="' . theme_image("download.png") . '"></a> <a href="' . $agent64_stable_url . '"><b>' . _('Download') . ' v0.4.4 (64bit)</b></a>
                                    </td>
                                </tr>
                            </tbody>
                        </table>
                    </div>
                    <p>' . _('Note: Additional agent versions are available from the') . ' <a href="http://nsclient.org/download/" target="_blank">' . _('NSClient++ downloads page') . '</a>.</p>
                </td>
            </tr>
        </table>';
            break;

        case CONFIGWIZARD_MODE_GETOBJECTS:

            $hostname = grab_array_var($inargs, "hostname", "");
            $address = grab_array_var($inargs, "ip_address", "");
            $password = grab_array_var($inargs, "password", "");
            $hostaddress = $address;

            $services_serial = grab_array_var($inargs, "services_serial", "");
            $serviceargs_serial = grab_array_var($inargs, "serviceargs_serial", "");

            $services = json_decode(base64_decode($services_serial), true);
            $serviceargs = json_decode(base64_decode($serviceargs_serial), true);

            // save data for later use in re-entrance
            $meta_arr = array();
            $meta_arr["hostname"] = $hostname;
            $meta_arr["ip_address"] = $address;
            $meta_arr["password"] = $password;
            $meta_arr["services"] = $services;
            $meta_arr["serivceargs"] = $serviceargs;
            save_configwizard_object_meta($wizard_name, $hostname, "", $meta_arr);

            $objs = array();

            if (!host_exists($hostname)) {
                $objs[] = array(
                    "type" => OBJECTTYPE_HOST,
                    "use" => "xiwizard_nsclient_host",
                    "host_name" => $hostname,
                    "address" => $hostaddress,
                    "icon_image" => "nsclient.png",
                    "statusmap_image" => "nsclient.png",
                    "_xiwizard" => $wizard_name,
                );
            }

            // see which services we should monitor
            foreach ($services as $svc => $svcstate) {

                //echo "PROCESSING: $svc -> $svcstate<BR>\n";

                switch ($svc) {

                    case "ping":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Ping",
                            "use" => "xiwizard_nsclient_ping_service",
                            "_xiwizard" => $wizard_name,
                        );
                        break;

                    case "cpu":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "CPU Usage",
                            "use" => "xiwizard_nsclient_service",
                            "check_command" => "check_xi_service_nsclient!" . $password . "!CPULOAD!-l 5," . $serviceargs["cpu_warning"] . "," . $serviceargs["cpu_critical"],
                            "_xiwizard" => $wizard_name,
                        );
                        break;

                    case "memory":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Memory Usage",
                            "use" => "xiwizard_nsclient_service",
                            "check_command" => "check_xi_service_nsclient!" . $password . "!MEMUSE!-w " . $serviceargs["memory_warning"] . " -c " . $serviceargs["memory_critical"],
                            "_xiwizard" => $wizard_name,
                        );
                        break;

                    case "uptime":
                        $objs[] = array(
                            "type" => OBJECTTYPE_SERVICE,
                            "host_name" => $hostname,
                            "service_description" => "Uptime",
                            "use" => "xiwizard_nsclient_service",
                            "check_command" => "check_xi_service_nsclient!" . $password . "!UPTIME",
                            "_xiwizard" => $wizard_name,
                        );
                        break;

                    case "disk":
                        $donedisks = array();
                        $diskid = 0;
                        foreach ($serviceargs["disk"] as $diskname) {

                            if ($diskname == "")
                                continue;

                            //echo "HANDLING DISK: $diskname<BR>";

                            // we already configured this disk
                            if (in_array($diskname, $donedisks))
                                continue;
                            $donedisks[] = $diskname;

                            $objs[] = array(
                                "type" => OBJECTTYPE_SERVICE,
                                "host_name" => $hostname,
                                "service_description" => "Drive " . $diskname . ": Disk Usage",
                                "use" => "xiwizard_nsclient_service",
                                "check_command" => "check_xi_service_nsclient!" . $password . "!USEDDISKSPACE!-l " . $diskname . " -w " . $serviceargs["disk_warning"][$diskid] . " -c " . $serviceargs["disk_critical"][$diskid],
                                "_xiwizard" => $wizard_name,
                            );

                            $diskid++;
                        }
                        break;

                    case "servicestate":

                        $enabledservices = $svcstate;
                        foreach ($enabledservices as $sid => $sstate) {

                            $sname = $serviceargs["servicestate"][$sid]["service"];
                            $sdesc = $serviceargs["servicestate"][$sid]["name"];

                            $objs[] = array(
                                "type" => OBJECTTYPE_SERVICE,
                                "host_name" => $hostname,
                                "service_description" => $sdesc,
                                "use" => "xiwizard_nsclient_service",
                                "check_command" => "check_xi_service_nsclient!" . $password . "!SERVICESTATE!-l " . $sname . " -d SHOWALL",
                                "_xiwizard" => $wizard_name,
                            );
                        }
                        break;

                    case "processstate":

                        $enabledprocs = $svcstate;
                        foreach ($enabledprocs as $pid => $pstate) {

                            $pname = $serviceargs["processstate"][$pid]["process"];
                            $pdesc = $serviceargs["processstate"][$pid]["name"];

                            $objs[] = array(
                                "type" => OBJECTTYPE_SERVICE,
                                "host_name" => $hostname,
                                "service_description" => $pdesc,
                                "use" => "xiwizard_nsclient_service",
                                "check_command" => "check_xi_service_nsclient!" . $password . "!PROCSTATE!-l " . $pname . " -d SHOWALL",
                                "_xiwizard" => $wizard_name,
                            );
                        }
                        break;

                    case "counter":

                        $enabledcounters = $svcstate;
                        foreach ($enabledcounters as $cid => $cstate) {

                            $cname = $serviceargs["counter"][$cid]["counter"];
                            $cdesc = $serviceargs["counter"][$cid]["name"];
                            $cformat = $serviceargs["counter"][$cid]["format"];
                            $cwarn = $serviceargs["counter"][$cid]["warning"];
                            $ccrit = $serviceargs["counter"][$cid]["critical"];

                            $checkcommand = "check_xi_service_nsclient!" . $password . "!COUNTER!-l \"" . $cname . "\"";
                            if ($cformat != "")
                                $checkcommand .= ",\"" . $cformat . "\"";
                            if ($cwarn != "")
                                $checkcommand .= " -w " . $cwarn;
                            if ($ccrit != "")
                                $checkcommand .= " -c " . $ccrit;

                            $objs[] = array(
                                "type" => OBJECTTYPE_SERVICE,
                                "host_name" => $hostname,
                                "service_description" => $cdesc,
                                "use" => "xiwizard_nsclient_service",
                                "check_command" => $checkcommand,
                                "_xiwizard" => $wizard_name,
                            );
                        }
                        break;

                    default:
                        break;
                }
            }

            $outargs[CONFIGWIZARD_NAGIOS_OBJECTS] = $objs;

            break;

        default:
            break;
    }

    return $output;
}
